"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerGuiCommands = void 0;
const pm2_1 = __importDefault(require("pm2"));
const path = require("path");
const pm2_2 = require("./pm2");
const deepmerge_1 = __importDefault(require("deepmerge"));
const default_gui_config_1 = require("./config/default-gui-config");
const fs_1 = __importDefault(require("fs"));
const yaml = __importStar(require("js-yaml"));
const cryptoShardus = __importStar(require("@shardeum-foundation/lib-crypto-utils"));
const project_data_1 = require("./utils/project-data");
const utils_1 = require("./utils");
const crypto_1 = __importDefault(require("crypto"));
const ajv_1 = __importDefault(require("ajv"));
const argon2_1 = __importDefault(require("argon2"));
let config = default_gui_config_1.defaultGuiConfig;
const validateGuiConfig = new ajv_1.default().compile(default_gui_config_1.guiConfigSchema);
cryptoShardus.init('64f152869ca2d473e4ba64ab53f49ccdb2edae22da192c126850970e788af347');
function isNumber(n) {
    const parsedN = parseInt(n);
    return !isNaN(parsedN) && isFinite(parsedN);
}
function validPassword(password) {
    return (password.length >= 8 &&
        password.length <= 128 &&
        /[A-Z]/.test(password) &&
        /[a-z]/.test(password) &&
        /[0-9]/.test(password) &&
        /[!@#$%^&*()_+*$]/.test(password));
}
const guiConfigPath = path.join(__dirname, `../${utils_1.File.GUI_CONFIG}`);
if (fs_1.default.existsSync(guiConfigPath)) {
    // eslint-disable-line security/detect-non-literal-fs-filename
    // eslint-disable-next-line security/detect-non-literal-fs-filename
    const fileConfig = JSON.parse(fs_1.default.readFileSync(guiConfigPath).toString());
    if (validateGuiConfig(fileConfig)) {
        config = (0, deepmerge_1.default)(config, fileConfig, {
            arrayMerge: (target, source) => source,
        });
        // `as guiConfigType` above is valid because validateGuiConfig() passed
    }
    else {
        console.warn(`warning: config has been ignored due to invalid JSON schema:`);
        console.warn(`${guiConfigPath}`);
    }
}
function registerGuiCommands(program) {
    const gui = program.command('gui').description('GUI related commands');
    gui
        .command('status')
        .description('Show if GUI is running or not; also the port and URL to connect to it')
        .action(() => {
        pm2_1.default.describe('operator-gui', (err, descriptions) => {
            if (err) {
                console.error(err);
                return pm2_1.default.disconnect();
            }
            if (descriptions.length === 0) {
                console.log('operator gui not running!');
                return pm2_1.default.disconnect();
            }
            const description = descriptions[0];
            const status = (0, pm2_2.statusFromPM2)(description);
            status.link = `https://localhost:${config.gui.port}/`;
            console.log(yaml.dump(status));
            return pm2_1.default.disconnect();
        });
    });
    gui
        .command('start')
        .description('Starts the GUI server')
        .action(async () => await startGui());
    gui
        .command('restart')
        .description('Restarts the GUI server')
        .action(async () => {
        await stopGui();
        await startGui();
    });
    gui
        .command('version')
        .description('Show the GUI version')
        .action(() => {
        console.log((0, project_data_1.getInstalledGuiVersion)());
    });
    gui
        .command('stop')
        .description('Stops the GUI server')
        .action(async () => {
        await stopGui();
    });
    const setCommand = gui
        .command('set')
        .description('command to set various config parameters');
    setCommand
        .command('port')
        .arguments('<port>')
        .description('Set the GUI server port')
        .action(port => {
        if (!isNumber(port)) {
            console.error('Port is not a number');
            return;
        }
        port = parseInt(port);
        if (port < 1024) {
            console.error('Port is reserved');
            return;
        }
        config.gui.port = parseInt(port);
        process.env.DASHPORT = port; // set the DASHPORT environment variable
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        fs_1.default.writeFile(path.join(__dirname, `../${utils_1.File.GUI_CONFIG}`), JSON.stringify(config, undefined, 2), err => {
            if (err)
                console.log(err);
        });
    });
    setCommand
        .command('password')
        .arguments('<password>')
        .description('Set the GUI server password, requirements: min 8 characters, max 128 characters, at least 1 lower case letter, at least 1 upper case letter, at least 1 number, at least 1 special character !@#$%^&*()_+*$')
        .option('-h', 'Changes how the password is hashed. For internal use only')
        .action(async (password, options) => {
        if (!options.h) {
            if (!validPassword(password)) {
                console.error('Invalid password: requirements: min 8 characters, at least 1 lower case letter, at least 1 upper case letter, at least 1 number, at least 1 special character !@#$%^&*()_+*$');
                return;
            }
            password = crypto_1.default.createHash('sha256').update(password).digest('hex');
        }
        config.gui.pass = await argon2_1.default.hash(password);
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        fs_1.default.writeFile(path.join(__dirname, `../${utils_1.File.GUI_CONFIG}`), JSON.stringify(config, undefined, 2), err => {
            if (err)
                console.error(err);
        });
    });
    gui
        .command('login')
        .arguments('<password>')
        .description('verify GUI password')
        .action(async (password) => {
        try {
            // Verify the password with saved hash
            const isValid = await argon2_1.default.verify(config.gui.pass, password);
            if (!isValid) {
                console.log(yaml.dump({ login: 'unauthorized' }));
                return;
            }
            console.log(yaml.dump({ login: 'authorized' }));
        }
        catch (err) {
            console.error('Error during password verification:', err);
            console.log(yaml.dump({ login: 'unauthorized' })); // Fail-safe unauthorized output
        }
    });
    function startGui() {
        // Exec PM2 to start the GUI server
        return new Promise((resolve, reject) => pm2_1.default.connect(err => {
            if (err) {
                console.error(err);
                reject('Unable to connect to PM2');
            }
            // Start next.js front end on port 3000
            pm2_1.default.start({
                name: 'operator-gui',
                cwd: `${path.join(__dirname, '../../../gui')}`,
                script: 'npm',
                args: 'start',
                env: { PORT: `${config.gui.port}` },
            }, err => {
                if (err) {
                    console.error(err);
                    reject('Unable to start GUI');
                }
                pm2_1.default.disconnect();
                resolve();
            });
        }));
    }
    function stopGui() {
        return new Promise((resolve, reject) => {
            pm2_1.default.connect(err => {
                if (err) {
                    console.error(err);
                    reject('Unable to connect to PM2');
                }
                pm2_1.default.stop('operator-gui', err => {
                    if (err) {
                        console.log(err);
                        reject('Unable to stop gui');
                    }
                    pm2_1.default.disconnect();
                    resolve();
                });
                pm2_1.default.delete('operator-gui', err => {
                    if (err) {
                        console.log(err);
                        reject('Unable to delete gui');
                    }
                    pm2_1.default.disconnect();
                    resolve();
                });
            });
        });
    }
}
exports.registerGuiCommands = registerGuiCommands;
//# sourceMappingURL=gui-commands.js.map