"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerNodeCommands = void 0;
const pm2_1 = require("./pm2");
const pm2_2 = __importDefault(require("pm2"));
const yaml = __importStar(require("js-yaml"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const deepmerge_1 = __importDefault(require("deepmerge"));
const default_network_config_1 = require("./config/default-network-config");
const default_node_config_1 = require("./config/default-node-config");
const default_rpc_config_1 = require("./config/default-rpc-config");
const fs_1 = __importStar(require("fs"));
const ethers_1 = require("ethers");
const utils_1 = require("./utils");
const ethereumjs_util_1 = require("ethereumjs-util");
const logger_1 = __importDefault(require("./utils/logger"));
const net_1 = require("net");
const ajv_1 = __importDefault(require("ajv"));
const projectFlags_1 = require("./projectFlags");
let config = default_network_config_1.defaultNetworkConfig;
let nodeConfig = default_node_config_1.defaultNodeConfig;
let rpcServer = {
    url: process.env.RPC_SERVER_URL,
};
const validateNetworkConfig = new ajv_1.default().compile(default_network_config_1.networkConfigSchema);
const validateNodeConfig = new ajv_1.default().compile(default_node_config_1.nodeConfigSchema);
const validateRpcConfig = new ajv_1.default().compile(default_rpc_config_1.rpcConfigSchema);
const networkConfigPath = path_1.default.join(__dirname, `../${utils_1.File.CONFIG}`);
// eslint-disable-next-line security/detect-non-literal-fs-filename
if (fs_1.default.existsSync(networkConfigPath)) {
    // eslint-disable-line security/detect-non-literal-fs-filename
    // eslint-disable-next-line security/detect-non-literal-fs-filename
    const fileConfig = JSON.parse(fs_1.default.readFileSync(networkConfigPath).toString());
    if (validateNetworkConfig(fileConfig)) {
        // check IP formats
        const networkConfig = fileConfig;
        // `as networkConfigType` above is valid because validateNetworkConfig() passed
        let good = true;
        for (const archiver of networkConfig.server.p2p.existingArchivers) {
            if (!(0, net_1.isIP)(archiver.ip)) {
                console.warn(`warning: config has been ignored due to invalid IP address: ${archiver.ip}`);
                console.warn(`${networkConfigPath}`);
                good = false;
            }
        }
        if (good)
            config = (0, deepmerge_1.default)(config, networkConfig, {
                arrayMerge: (target, source) => source,
            });
    }
    else {
        console.warn(`warning: config has been ignored due to invalid JSON schema:`);
        console.warn(`${networkConfigPath}`);
    }
}
const nodeConfigPath = path_1.default.join(__dirname, `../${utils_1.File.NODE_CONFIG}`);
// eslint-disable-next-line security/detect-non-literal-fs-filename
if (fs_1.default.existsSync(nodeConfigPath)) {
    // eslint-disable-line security/detect-non-literal-fs-filename
    // eslint-disable-next-line security/detect-non-literal-fs-filename
    const fileConfig = JSON.parse(fs_1.default.readFileSync(nodeConfigPath).toString());
    if (validateNodeConfig(fileConfig)) {
        nodeConfig = (0, deepmerge_1.default)(nodeConfig, fileConfig, {
            arrayMerge: (target, source) => source,
        });
        // `as nodeConfigType` above is valid because validateNodeConfig() passed
    }
    else {
        console.warn(`warning: config has been ignored due to invalid JSON schema:`);
        console.warn(`${nodeConfigPath}`);
    }
}
else {
    // eslint-disable-next-line security/detect-non-literal-fs-filename
    fs_1.default.writeFileSync(path_1.default.join(__dirname, `../${utils_1.File.NODE_CONFIG}`), JSON.stringify(nodeConfig, undefined, 2), { encoding: 'utf8', mode: 0o600 });
}
const rpcConfigPath = path_1.default.join(__dirname, `../${utils_1.File.RPC_SERVER}`);
// eslint-disable-next-line security/detect-non-literal-fs-filename
if (fs_1.default.existsSync(rpcConfigPath)) {
    // eslint-disable-line security/detect-non-literal-fs-filename
    // eslint-disable-next-line security/detect-non-literal-fs-filename
    const fileConfig = JSON.parse(fs_1.default.readFileSync(rpcConfigPath).toString());
    if (validateRpcConfig(fileConfig)) {
        rpcServer = (0, deepmerge_1.default)(rpcServer, fileConfig, {
            arrayMerge: (target, source) => source,
        });
        // `as rpcConfigType` above is valid because validateRpcConfig() passed
    }
    else {
        console.warn(`warning: config has been ignored due to invalid JSON schema:`);
        console.warn(`${rpcConfigPath}`);
    }
}
if (process.env.APP_SEEDLIST) {
    config = (0, deepmerge_1.default)(config, {
        server: {
            p2p: {
                existingArchivers: [
                    {
                        ip: process.env.APP_SEEDLIST,
                        port: 4000,
                        publicKey: '758b1c119412298802cd28dbfa394cdfeecc4074492d60844cc192d632d84de3',
                    },
                ],
            },
        },
    }, { arrayMerge: (target, source) => source });
}
if (process.env.EXISTING_ARCHIVERS) {
    const existingArchivers = JSON.parse(process.env.EXISTING_ARCHIVERS);
    if (existingArchivers.length > 0) {
        config = (0, deepmerge_1.default)(config, {
            server: {
                p2p: {
                    existingArchivers,
                },
            },
        }, { arrayMerge: (target, source) => source });
    }
}
if (process.env.APP_MONITOR) {
    config = (0, deepmerge_1.default)(config, {
        server: {
            reporting: {
                recipient: `http://${process.env.APP_MONITOR}:3000/api`,
            },
        },
    }, { arrayMerge: (target, source) => source });
}
if (process.env.EXT_IP) {
    config = (0, deepmerge_1.default)(config, {
        server: {
            ip: {
                externalIp: process.env.EXT_IP === 'auto'
                    ? process.env.SERVERIP
                    : process.env.EXT_IP,
            },
        },
    }, { arrayMerge: (target, source) => source });
}
if (process.env.INT_IP) {
    config = (0, deepmerge_1.default)(config, {
        server: {
            ip: {
                internalIp: process.env.INT_IP === 'auto'
                    ? process.env.SERVERIP
                    : process.env.INT_IP,
            },
        },
    }, { arrayMerge: (target, source) => source });
}
if (process.env.SHMEXT) {
    config = (0, deepmerge_1.default)(config, {
        server: {
            ip: {
                externalPort: parseInt(process.env.SHMEXT),
            },
        },
    });
}
if (process.env.SHMINT) {
    config = (0, deepmerge_1.default)(config, {
        server: {
            ip: {
                internalPort: parseInt(process.env.SHMINT),
            },
        },
    });
}
const dashboardPackageJson = JSON.parse((0, fs_1.readFileSync)(path_1.default.join(__dirname, '../../package.json'), 'utf8'));
// eslint-disable-next-line security/detect-non-literal-fs-filename
fs_1.default.writeFileSync(path_1.default.join(__dirname, `../${utils_1.File.CONFIG}`), JSON.stringify(config, undefined, 2));
function registerNodeCommands(program) {
    program
        .command('status')
        .description('Show if validator is running or not; also the port and URL to connect to it')
        .action(async () => {
        pm2_2.default.describe('validator', async (err, descriptions) => {
            // PM2 not reachable
            if (err) {
                console.error(err);
                return pm2_2.default.disconnect();
            }
            let publicKey = '';
            // Fetch the public key from secrets.json if it exists
            // eslint-disable-next-line security/detect-non-literal-fs-filename
            if (fs_1.default.existsSync(path_1.default.join(__dirname, `../${utils_1.File.SECRETS}`))) {
                const secrets = JSON.parse(
                // eslint-disable-next-line security/detect-non-literal-fs-filename
                fs_1.default
                    .readFileSync(path_1.default.join(__dirname, `../${utils_1.File.SECRETS}`))
                    .toString());
                publicKey = secrets.publicKey;
            }
            const [{ stakeRequired }, performance, { totalTimeValidating, lastRotationIndex, lastActive }, { exitMessage, exitStatus }, accountInfo,] = await Promise.all([
                (0, utils_1.fetchStakeParameters)(config),
                (0, utils_1.getPerformanceStatus)(),
                (0, utils_1.fetchNodeProgress)().then(utils_1.getProgressData),
                (0, utils_1.getExitInformation)(),
                (0, utils_1.getAccountInfoParams)(config, publicKey),
            ]);
            // TODO: Use Promise.allSettled. Need to update nodeJs to 12.9
            let nodeInfo;
            try {
                nodeInfo = await (0, utils_1.fetchNodeInfo)(config);
            }
            catch (e) {
                logger_1.default.error('Unable to fetch node info: ' + e);
                nodeInfo = null;
            }
            let unstakable = {
                unlocked: false,
                reason: 'Could not fetch data',
                remainingTime: -1,
            };
            if (descriptions.length === 0) {
                if (accountInfo.nominator) {
                    const eoaData = await (0, utils_1.fetchEOADetails)(config, accountInfo.nominator);
                    if (eoaData) {
                        const unstakableData = await (0, utils_1.fetchUnstakeableDetails)(config, publicKey, accountInfo.nominator);
                        unstakable = unstakableData !== null && unstakableData !== void 0 ? unstakableData : unstakable;
                    }
                }
                // Node process not started
                console.log(yaml.dump({
                    state: 'stopped',
                    exitMessage,
                    exitStatus,
                    performance,
                    stakeRequirement: stakeRequired
                        ? ethers_1.ethers.utils.formatEther(stakeRequired)
                        : '',
                    lockedStake: accountInfo.lockedStake
                        ? ethers_1.ethers.utils.formatEther(accountInfo.lockedStake)
                        : '',
                    totalPenalty: accountInfo.totalPenalty
                        ? ethers_1.ethers.utils.formatEther(accountInfo.totalPenalty)
                        : '',
                    autorestart: nodeConfig.autoRestart,
                    stakeState: unstakable,
                }));
                utils_1.cache.writeMaps();
                return pm2_2.default.disconnect();
            }
            const description = descriptions[0];
            const status = (0, pm2_1.statusFromPM2)(description);
            if (status.status !== 'stopped') {
                // Node is started and active
                const lockedStakeStr = accountInfo.lockedStake
                    ? ethers_1.ethers.utils.formatEther(accountInfo.lockedStake)
                    : '';
                let nodeStatus = nodeInfo.status;
                if (nodeStatus === 'initializing')
                    nodeStatus =
                        lockedStakeStr === '0.0' ? 'need-stake' : 'waiting-for-network';
                if (nodeStatus === 'active') {
                    unstakable = {
                        unlocked: false,
                        reason: 'Node is active',
                        remainingTime: -1,
                    };
                }
                else {
                    if (accountInfo.nominator) {
                        const eoaData = await (0, utils_1.fetchEOADetails)(config, accountInfo.nominator);
                        if (eoaData) {
                            const unstakableData = await (0, utils_1.fetchUnstakeableDetails)(config, publicKey, accountInfo.nominator);
                            unstakable = unstakableData !== null && unstakableData !== void 0 ? unstakableData : unstakable;
                        }
                    }
                }
                console.log(yaml.dump({
                    state: nodeStatus,
                    exitMessage,
                    exitStatus,
                    totalTimeRunning: status.uptimeInSeconds,
                    totalTimeValidating: totalTimeValidating,
                    lastActive: lastActive,
                    lastRotationIndex: lastRotationIndex,
                    stakeRequirement: stakeRequired
                        ? ethers_1.ethers.utils.formatEther(stakeRequired)
                        : '',
                    nominatorAddress: accountInfo.nominator,
                    nomineeAddress: publicKey,
                    performance,
                    currentRewards: ethers_1.ethers.utils.formatEther(accountInfo.accumulatedRewards.toString()),
                    lockedStake: lockedStakeStr,
                    totalPenalty: accountInfo.totalPenalty
                        ? ethers_1.ethers.utils.formatEther(accountInfo.totalPenalty)
                        : '',
                    autorestart: nodeConfig.autoRestart,
                    nodeInfo: nodeInfo,
                    stakeState: unstakable,
                    // TODO: Add fetching node info when in standby
                }));
                utils_1.cache.writeMaps();
                return pm2_2.default.disconnect();
            }
            // Node was started but is currently inactive
            if (accountInfo.nominator) {
                const eoaData = await (0, utils_1.fetchEOADetails)(config, accountInfo.nominator);
                if (eoaData) {
                    const unstakableData = await (0, utils_1.fetchUnstakeableDetails)(config, publicKey, accountInfo.nominator);
                    unstakable = unstakableData !== null && unstakableData !== void 0 ? unstakableData : unstakable;
                }
            }
            console.log(yaml.dump({
                state: 'stopped',
                exitMessage,
                exitStatus,
                performance,
                stakeRequirement: stakeRequired
                    ? ethers_1.ethers.utils.formatEther(stakeRequired)
                    : '',
                lockedStake: accountInfo.lockedStake
                    ? ethers_1.ethers.utils.formatEther(accountInfo.lockedStake)
                    : '',
                nominatorAddress: accountInfo.nominator,
                currentRewards: accountInfo
                    ? ethers_1.ethers.utils.formatEther(accountInfo.accumulatedRewards.toString())
                    : '',
                totalPenalty: accountInfo.totalPenalty
                    ? ethers_1.ethers.utils.formatEther(accountInfo.totalPenalty)
                    : '',
                autorestart: nodeConfig.autoRestart,
                stakeState: unstakable,
            }));
            utils_1.cache.writeMaps();
            return pm2_2.default.disconnect();
        });
    });
    program
        .command('stake_info')
        .description('Show staking info for a particular EOA account')
        .argument('<address>', 'The EOA address to fetch stake info for')
        .action(async (address) => {
        var _a, _b, _c, _d, _e, _f;
        if (!ethers_1.ethers.utils.isAddress(address)) {
            console.error('Invalid address entered');
            return;
        }
        try {
            const eoaData = await (0, utils_1.fetchEOADetails)(config, address);
            const stakeValue = (_b = (_a = eoaData === null || eoaData === void 0 ? void 0 : eoaData.operatorAccountInfo) === null || _a === void 0 ? void 0 : _a.stake) === null || _b === void 0 ? void 0 : _b.value;
            const nominee = (_d = (_c = eoaData === null || eoaData === void 0 ? void 0 : eoaData.operatorAccountInfo) === null || _c === void 0 ? void 0 : _c.nominee) !== null && _d !== void 0 ? _d : '';
            let nodeAccountData;
            if (((_e = eoaData === null || eoaData === void 0 ? void 0 : eoaData.operatorAccountInfo) === null || _e === void 0 ? void 0 : _e.nominee) != null) {
                nodeAccountData = await (0, utils_1.fetchEOADetails)(config, eoaData.operatorAccountInfo.nominee);
            }
            // Convert stake value to ether, handling potential hexadecimal input
            const stakeOutput = stakeValue
                ? ethers_1.ethers.utils.formatEther(ethers_1.ethers.BigNumber.from(stakeValue.startsWith('0x') ? stakeValue : '0x' + stakeValue).toString())
                : '';
            console.log(yaml.dump({
                stake: stakeOutput,
                nominee: nominee,
                rewards: ((_f = nodeAccountData === null || nodeAccountData === void 0 ? void 0 : nodeAccountData.reward) === null || _f === void 0 ? void 0 : _f.value)
                    ? ethers_1.ethers.utils.formatEther(new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(nodeAccountData.reward.value), 16).toString())
                    : undefined,
            }));
        }
        catch (error) {
            console.log(error);
            console.error(`Error fetching stake details for ${address}: ${error}`);
        }
    });
    program
        .command('start')
        .description('Starts the validator')
        .action(() => {
        // Run the validators clean script
        (0, child_process_1.execFile)(`node`, [`${path_1.default.join(__dirname, projectFlags_1.VALIDATOR_CLEAN_PATH)}`], () => {
            // Exec PM2 to start the shardeum validator
            pm2_2.default.connect(err => {
                if (err) {
                    console.error(err);
                    throw 'Unable to connect to PM2';
                }
                const scriptPath = path_1.default.join(__dirname, `../../${utils_1.File.ENVIRONMENT_CONFIG}`);
                pm2_2.default.start(scriptPath, err => {
                    if (err)
                        throw err;
                    pm2_2.default.describe('validator', (err, processDescription) => {
                        var _a, _b;
                        if (err) {
                            pm2_2.default.disconnect();
                            throw err;
                        }
                        if (processDescription && processDescription.length > 0) {
                            const processInfo = processDescription[0];
                            if (((_a = processInfo.pm2_env) === null || _a === void 0 ? void 0 : _a.status) === 'online') {
                                console.log('Node process started');
                            }
                            else {
                                throw new Error(`The process is not running. Status: ${(_b = processInfo.pm2_env) === null || _b === void 0 ? void 0 : _b.status}`);
                            }
                        }
                        else {
                            throw new Error("Node process wasn't started");
                        }
                        pm2_2.default.disconnect();
                    });
                });
            });
        });
    });
    program
        .command('stop')
        .description('Stops the validator')
        .option('-f, --force', 'stops the node without prompting for confirmation even if it is participating and could get slashed')
        .action(options => {
        function stopNode() {
            pm2_2.default.stop('validator', err => {
                if (err) {
                    console.error(err);
                }
                else {
                    nodeConfig.lastStopped = new Date().getTime();
                    console.log('Node process stopped', nodeConfig);
                    writeNodeConfig();
                }
                return pm2_2.default.disconnect();
            });
        }
        // Exec PM2 to stop the shardeum validator
        pm2_2.default.connect(err => {
            if (err) {
                console.error(err);
                throw 'Unable to connect to PM2';
            }
            pm2_2.default.describe('validator', async (err, descriptions) => {
                if (descriptions.length === 0) {
                    console.error('Node is not running');
                    return pm2_2.default.disconnect();
                }
                const description = descriptions[0];
                const status = (0, pm2_1.statusFromPM2)(description);
                if (status.status === 'stopped') {
                    console.error('Node is not running');
                    return pm2_2.default.disconnect();
                }
                if (!options.force) {
                    let nodeInfo;
                    try {
                        nodeInfo = await (0, utils_1.fetchNodeInfo)(config);
                    }
                    catch (error) {
                        const answer = await (0, utils_1.getUserInput)('Unable to verify node status. If the node is active, stopping it could result in losing the stake amount. ' +
                            'Confirm if you would like to force the node to stop (y/N): ');
                        if (answer.toLowerCase() === 'y') {
                            return stopNode();
                        }
                        return pm2_2.default.disconnect();
                    }
                    if (nodeInfo != null &&
                        (nodeInfo.status === 'syncing' || nodeInfo.status === 'active')) {
                        const answer = await (0, utils_1.getUserInput)('The node is active and stopping it could result in losing the stake amount. ' +
                            'Confirm if you would like to force the node to stop (y/N): ');
                        if (answer.toLowerCase() === 'y') {
                            return stopNode();
                        }
                        return pm2_2.default.disconnect();
                    }
                }
                return stopNode();
            });
        });
    });
    program
        .command('stake')
        .argument('<value>', 'The amount of SHM to stake')
        .description('Stake the set amount of SHM at the stake address. Rewards will be sent to set reward address.')
        .action(async (stakeValue) => {
        //TODO should we handle consecutive stakes?
        // Fetch the public key from secrets.json
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        if (!fs_1.default.existsSync(path_1.default.join(__dirname, `../${utils_1.File.SECRETS}`))) {
            console.error('Please start the node once before staking');
            return;
        }
        const secrets = JSON.parse(
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        fs_1.default.readFileSync(path_1.default.join(__dirname, `../${utils_1.File.SECRETS}`)).toString());
        if (secrets.publicKey == null) {
            console.error('Unable to find public key in secrets.json');
            return;
        }
        // Take input from user for PRIVATE KEY
        let privateKey = await (0, utils_1.getUserInput)('Please enter your private key: ');
        while (privateKey.length !== 64 ||
            !(0, ethereumjs_util_1.isValidPrivate)(Buffer.from(privateKey, 'hex'))) {
            console.log('Invalid private key entered.');
            privateKey = await (0, utils_1.getUserInput)('Please enter your private key: ');
        }
        const provider = new ethers_1.ethers.providers.JsonRpcProvider(rpcServer.url);
        const walletWithProvider = new ethers_1.ethers.Wallet(privateKey, provider);
        const [{ stakeRequired }, eoaData] = await Promise.all([
            (0, utils_1.fetchStakeParameters)(config),
            (0, utils_1.fetchEOADetails)(config, walletWithProvider.address),
        ]);
        if (ethers_1.ethers.BigNumber.from(stakeRequired).gt(ethers_1.ethers.utils.parseEther(stakeValue))) {
            if (eoaData == null) {
                /*prettier-ignore*/
                console.error(`Stake amount must be greater than ${ethers_1.ethers.utils.formatEther(stakeRequired)} SHM`);
                return;
            }
        }
        try {
            const [gasPrice, from, nonce] = await Promise.all([
                walletWithProvider.getGasPrice(),
                walletWithProvider.getAddress(),
                walletWithProvider.getTransactionCount(),
            ]);
            const stakeData = {
                isInternalTx: true,
                internalTXType: 6,
                nominator: walletWithProvider.address.toLowerCase(),
                timestamp: Date.now(),
                nominee: secrets.publicKey,
                stake: ethers_1.ethers.utils.parseEther(stakeValue).toString(),
            };
            const value = ethers_1.ethers.BigNumber.from(stakeData.stake);
            console.log(stakeData);
            const txDetails = {
                from,
                to: '0x0000000000000000000000000000000000010000',
                gasPrice,
                gasLimit: 30000000,
                value,
                data: ethers_1.ethers.utils.hexlify(ethers_1.ethers.utils.toUtf8Bytes(JSON.stringify(stakeData))),
                nonce,
            };
            const { hash, data, wait } = await walletWithProvider.sendTransaction(txDetails);
            console.log('TX RECEIPT: ', { hash, data });
            const txConfirmation = await wait();
            console.log('TX CONFRIMED: ', txConfirmation);
        }
        catch (error) {
            console.error(error);
        }
    });
    async function unstake(options) {
        var _a, _b, _c, _d, _e;
        // Take input from user for PRIVATE KEY
        let privateKey = await (0, utils_1.getUserInput)('Please enter your private key: ');
        while (privateKey.length !== 64 ||
            !(0, ethereumjs_util_1.isValidPrivate)(Buffer.from(privateKey, 'hex'))) {
            console.log('Invalid private key entered.');
            privateKey = await (0, utils_1.getUserInput)('Please enter your private key: ');
        }
        try {
            const provider = new ethers_1.ethers.providers.JsonRpcProvider(rpcServer.url);
            const walletWithProvider = new ethers_1.ethers.Wallet(privateKey, provider);
            const eoaData = await (0, utils_1.fetchEOADetails)(config, walletWithProvider.address);
            if (!eoaData) {
                console.error("Couldn't unstake (`eoaData` is null)");
                return;
            }
            if (((_a = eoaData.operatorAccountInfo) === null || _a === void 0 ? void 0 : _a.nominee) == null ||
                ((_c = (_b = eoaData.operatorAccountInfo) === null || _b === void 0 ? void 0 : _b.stake) === null || _c === void 0 ? void 0 : _c.value) === '00') {
                console.error('No stake found');
                return;
            }
            const [gasPrice, from, nonce] = await Promise.all([
                walletWithProvider.getGasPrice(),
                walletWithProvider.getAddress(),
                walletWithProvider.getTransactionCount(),
            ]);
            const unstakeData = {
                isInternalTx: true,
                internalTXType: 7,
                nominator: walletWithProvider.address.toLowerCase(),
                timestamp: Date.now(),
                nominee: (_d = eoaData === null || eoaData === void 0 ? void 0 : eoaData.operatorAccountInfo) === null || _d === void 0 ? void 0 : _d.nominee,
                force: (_e = options.force) !== null && _e !== void 0 ? _e : false,
            };
            console.log(unstakeData);
            const txDetails = {
                from,
                to: '0x0000000000000000000000000000000000010000',
                gasPrice,
                gasLimit: 30000000,
                data: ethers_1.ethers.utils.hexlify(ethers_1.ethers.utils.toUtf8Bytes(JSON.stringify(unstakeData))),
                nonce,
            };
            console.log(txDetails);
            const { hash, data, wait } = await walletWithProvider.sendTransaction(txDetails);
            console.log('TX RECEIPT: ', { hash, data });
            const txConfirmation = await wait();
            console.log('TX CONFRIMED: ', txConfirmation);
        }
        catch (error) {
            console.error(error);
        }
    }
    program
        .command('unstake')
        .description('Remove staked SHM')
        .option('-f, --force', 'Force unstake in case the node is stuck, will forfeit rewards')
        .action(async (options) => {
        try {
            if (options.force) {
                const answer = await (0, utils_1.getUserInput)('Node is currently participating in the network, unstaking could result in a penalty. ' +
                    'Confirm if you would like to force unstake (y/N): ');
                if (answer.toLowerCase() !== 'y') {
                    return;
                }
            }
            else {
                let nodeInfo;
                try {
                    nodeInfo = await (0, utils_1.fetchNodeInfo)(config);
                }
                catch (error) {
                    // Error while fetching nodeInfo - presuming node is not active
                }
                const nodeStatus = nodeInfo === null || nodeInfo === void 0 ? void 0 : nodeInfo.status;
                if (nodeStatus === 'initializing' ||
                    nodeStatus === 'standby' ||
                    nodeStatus === 'syncing' ||
                    nodeStatus === 'active') {
                    throw 'Please stop your node before unstaking.';
                }
            }
            await unstake(options);
        }
        catch (error) {
            console.error(error);
        }
    });
    program
        .command('update')
        .description('Update the CLI and the GUI')
        .action(() => {
        (0, child_process_1.exec)('sh update.sh', { cwd: path_1.default.join(__dirname, '../..') }, (error, stdout, stderr) => {
            console.log(stdout);
            console.log(stderr);
            if (error != null) {
                console.log(`exec error: ${error}`);
            }
        });
        (0, child_process_1.exec)('sh update.sh', { cwd: path_1.default.join(__dirname, '../../../gui') }, (error, stdout, stderr) => {
            console.log(stdout);
            console.log(stderr);
            if (error != null) {
                console.log(`exec error: ${error}`);
            }
        });
    });
    program
        .command('version')
        .description('Shows the installed version, latest version and minimum version of the operator dashboard')
        .action(async () => {
        const validatorVersions = await (0, utils_1.fetchValidatorVersions)(config);
        if (!validatorVersions) {
            throw new Error("Couldn't fetch validator versions");
        }
        let versions = {
            runningCliVersion: dashboardPackageJson.version,
            runningCliBranch: await (0, utils_1.getBranchNameForCLI)(),
            runningCliCommitHash: await (0, utils_1.getCommitHashForCLI)(),
            minimumCliVersion: '0.1.0',
            latestCliVersion: await (0, utils_1.getLatestCliVersion)(),
            minShardeumVersion: validatorVersions.minVersion,
            activeShardeumVersion: validatorVersions.activeVersion,
        };
        if ((0, utils_1.isGuiInstalled)()) {
            versions = {
                ...versions,
                runningGuiVersion: (0, utils_1.getInstalledGuiVersion)(),
                runningGuiBranch: await (0, utils_1.getBranchNameForGUI)(),
                runningGuiBranchCommitHash: await (0, utils_1.getCommitHashForGUI)(),
                minimumGuiVersion: '0.1.0',
                latestGuiVersion: await (0, utils_1.getLatestGuiVersion)(),
            };
        }
        if ((0, utils_1.isValidatorInstalled)()) {
            versions = {
                ...versions,
                runnningValidatorVersion: (0, utils_1.getInstalledValidatorVersion)(),
                runningValidatorBranch: await (0, utils_1.getBranchNameForValidator)(),
                runningValidatorBranchCommitHash: await (0, utils_1.getCommitHashForValidator)(),
            };
        }
        console.log(yaml.dump(versions));
    });
    program
        .command('network-stats')
        .description('Show statistics like TPS, active nodes etc. about the network')
        .action(async () => {
        pm2_2.default.describe('validator', async (err, [descriptions]) => {
            if (!err) {
                const networkStats = await (0, utils_1.getNetworkParams)(config, descriptions);
                console.log(yaml.dump(networkStats));
            }
            pm2_2.default.disconnect();
        });
    });
    program
        .command('node-settings')
        .description('Display node settings')
        .action(() => {
        const settings = (0, utils_1.getNodeSettings)();
        console.log(yaml.dump({
            autoRestart: settings.autoRestart,
            lastStopped: settings.lastStopped,
        }));
    });
    const setCommand = program
        .command('set')
        .description('command to set various config parameters');
    // setCommand
    //   .command('external_port')
    //   .arguments('<port>')
    //   .description('Set the external port for the validator')
    //   .action(port => {
    //     config.server.ip.externalPort = parseInt(port);
    //     // eslint-disable-next-line security/detect-non-literal-fs-filename
    //     fs.writeFile(
    //       path.join(__dirname, `../${File.CONFIG}`),
    //       JSON.stringify(config, undefined, 2),
    //       err => {
    //         if (err) console.error(err);
    //       }
    //     );
    //   });
    // setCommand
    //   .command('internal_port')
    //   .arguments('<port>')
    //   .description('Set the internal port for the validator')
    //   .action(port => {
    //     config.server.ip.internalPort = parseInt(port);
    //     // eslint-disable-next-line security/detect-non-literal-fs-filename
    //     fs.writeFile(
    //       path.join(__dirname, `../${File.CONFIG}`),
    //       JSON.stringify(config, undefined, 2),
    //       err => {
    //         if (err) console.error(err);
    //       }
    //     );
    //   });
    // setCommand
    //   .command('external_ip')
    //   .arguments('<ip>')
    //   .description('Set the external ip for the validator')
    //   .action(ip => {
    //     config.server.ip.externalIp = ip;
    //     // eslint-disable-next-line security/detect-non-literal-fs-filename
    //     fs.writeFile(
    //       path.join(__dirname, `../${File.CONFIG}`),
    //       JSON.stringify(config, undefined, 2),
    //       err => {
    //         if (err) console.error(err);
    //       }
    //     );
    //   });
    // setCommand
    //   .command('internal_ip')
    //   .arguments('<ip>')
    //   .description('Set the internal ip for the validator')
    //   .action(ip => {
    //     config.server.ip.internalIp = ip;
    //     // eslint-disable-next-line security/detect-non-literal-fs-filename
    //     fs.writeFile(
    //       path.join(__dirname, `../${File.CONFIG}`),
    //       JSON.stringify(config, undefined, 2),
    //       err => {
    //         if (err) console.error(err);
    //       }
    //     );
    //   });
    setCommand
        .command('rpc_url')
        .argument('<url>')
        .description("Set the RPC server's URL")
        .action(url => {
        rpcServer.url = url;
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        fs_1.default.writeFile(path_1.default.join(__dirname, `../${utils_1.File.RPC_SERVER}`), JSON.stringify(rpcServer, undefined, 2), err => {
            if (err)
                console.error(err);
        });
    });
    setCommand
        .command('auto_restart')
        .argument('<true/false>')
        .description('To autostart the node after being rotated out. Set autostart to true or false')
        .action((autostart) => {
        const input = autostart.toLowerCase();
        if (input !== 'true' && input !== 'false') {
            console.error('Invalid input. Please enter true or false');
            return;
        }
        nodeConfig.autoRestart = input === 'true';
        writeNodeConfig();
    });
    function writeNodeConfig() {
        // eslint-disable-next-line security/detect-non-literal-fs-filename
        fs_1.default.writeFile(path_1.default.join(__dirname, `../${utils_1.File.NODE_CONFIG}`), JSON.stringify(nodeConfig, undefined, 2), { encoding: 'utf8', mode: 0o600 }, err => {
            if (err)
                console.error(err);
        });
    }
    // setCommand
    //   .command('archiver')
    //   .arguments('<URL>')
    //   .description('Set the URL for the archiver')
    //   .action(url => {
    //     //TODO interact with node
    //   });
}
exports.registerNodeCommands = registerNodeCommands;
//# sourceMappingURL=node-commands.js.map