"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchCycleDuration = exports.fetchStakeParameters = exports.getAccountInfoParams = exports.fetchValidatorVersions = exports.fetchUnstakeableDetails = exports.fetchEOADetails = exports.getNetworkParams = exports.fetchNodeInfo = exports.fetchInitialParameters = exports.getNewActiveNode = exports.networkAccount = exports.cache = void 0;
const axios_1 = __importStar(require("axios"));
const ethereumjs_util_1 = require("ethereumjs-util");
const pm2_1 = require("../pm2");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const tcache_1 = __importDefault(require("./tcache"));
const utils_1 = require("../utils");
exports.cache = new tcache_1.default();
exports.networkAccount = '1000000000000000000000000000000000000000000000000000000000000001';
let savedActiveNode = undefined;
/**
 * Checks if active node is already defined or present in file. Returns true
 * If active node is not available, returns false
 * denoting that new node needs to be fetched
 *
 * @returns {boolean} True if active node is already defined, false if not
 */
function readActiveNode() {
    // Check if savedActiveNode is already defined
    if (savedActiveNode !== undefined) {
        return true;
    }
    else {
        try {
            // eslint-disable-next-line security/detect-non-literal-fs-filename
            const data = fs_1.default.readFileSync(path_1.default.join(__dirname, `../../${utils_1.File.ACTIVE_NODE}`), 'utf8');
            savedActiveNode = JSON.parse(data);
            return true;
        }
        catch (e) {
            return false;
        }
    }
}
async function fetchDataFromNetwork(config, query, callback) {
    let retries = 5;
    if (!readActiveNode()) {
        await getNewActiveNode(config);
    }
    if (!savedActiveNode) {
        throw new Error('Unable to fetch active node');
    }
    let data = { data: null, status: 500 };
    let finalError = null;
    const url = `http://${savedActiveNode.ip}:${savedActiveNode.port}` + query;
    try {
        data = await axios_1.default.get(url, { timeout: 2000 });
    }
    catch (e) {
        if (e instanceof axios_1.AxiosError) {
            finalError = e;
        }
        // set data to null and status to 500 to indicate that the request failed
        data = { data: null, status: 500 };
    }
    while ((data.status === 500 || callback(data.data)) && retries--) {
        try {
            await getNewActiveNode(config);
        }
        catch (e) {
            continue;
        }
        // `savedActiveNode` may mutate between loop iterations, so `url` must be
        // set for each loop iteration
        const url = `http://${savedActiveNode.ip}:${savedActiveNode.port}` + query;
        try {
            data = await axios_1.default.get(url, { timeout: 2000 });
        }
        catch (e) {
            if (e instanceof axios_1.AxiosError) {
                finalError = e;
            }
            // set data to null and status to 500 to indicate that the request failed
            data = { data: null, status: 500 };
        }
    }
    if (retries < 0) {
        // figure out why we ran out of retries before throwing our error
        let reason = 'unknown reason';
        if (finalError != null) {
            if (finalError.response) {
                reason = `status ${finalError.response.status}`;
            }
            else if (finalError.request) {
                reason = 'no response received';
            }
            else {
                reason = `setting up request: ${finalError.message}`;
            }
        }
        throw new Error(`Unable to fetch data from network (out of retries: ${reason}).`);
    }
    else {
        return data.data;
    }
}
/**
 * Fetches a new random active node from the network.
 * Saves to file by overwriting the previous active node.
 * Also saves the active node to a global variable.
 *
 * @param config
 * @returns {Promise<void>}
 * @throws {Error} If unable to fetch list of nodes in the network
 */
async function getNewActiveNode(config) {
    const randomArchiver = config.server.p2p.existingArchivers[Math.floor(Math.random() * config.server.p2p.existingArchivers.length)];
    const archiverUrl = `http://${randomArchiver.ip}:${randomArchiver.port}/nodelist`;
    const nodeList = await axios_1.default
        .get(archiverUrl, { timeout: 2000 })
        .then(res => res.data)
        .catch(err => console.error(err));
    if (!(nodeList === null || nodeList === void 0 ? void 0 : nodeList.nodeList)) {
        throw new Error('Unable to fetch list of nodes in the network');
    }
    savedActiveNode =
        nodeList.nodeList[Math.floor(Math.random() * nodeList.nodeList.length)];
    //Write savedActiveNode to file
    // eslint-disable-next-line security/detect-non-literal-fs-filename
    fs_1.default.writeFileSync(path_1.default.join(__dirname, `../../${utils_1.File.ACTIVE_NODE}`), JSON.stringify(savedActiveNode));
}
exports.getNewActiveNode = getNewActiveNode;
async function fetchInitialParameters(config) {
    var _a, _b;
    const value = exports.cache.get('initialParameters');
    if (value) {
        const parsedValue = JSON.parse(value);
        if (parsedValue.nodeRewardAmount && parsedValue.nodeRewardInterval) {
            return {
                nodeRewardAmount: new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(parsedValue.nodeRewardAmount), 16),
                nodeRewardInterval: new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(parsedValue.nodeRewardInterval), 16),
            };
        }
    }
    const initialParams = await fetchDataFromNetwork(config, `/account/${exports.networkAccount}?type=5`, data => (data === null || data === void 0 ? void 0 : data.account) == null);
    const response = (_b = (_a = initialParams === null || initialParams === void 0 ? void 0 : initialParams.account) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.current;
    if (!response) {
        throw new Error("Fetched initial parameters, but account data isn't found");
    }
    const nodeRewardAmount = new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(response.nodeRewardAmountUsd), 16);
    const nodeRewardInterval = new ethereumjs_util_1.BN(response.nodeRewardInterval);
    if (!nodeRewardAmount || !nodeRewardInterval) {
        throw new Error('Fetched initial parameters, but nodeRewardAmount and nodeRewardInterval are not found');
    }
    const cycleDuration = await fetchCycleDuration(config);
    exports.cache.set('initialParameters', JSON.stringify({ nodeRewardAmount, nodeRewardInterval }), cycleDuration * 1000);
    return { nodeRewardAmount, nodeRewardInterval };
}
exports.fetchInitialParameters = fetchInitialParameters;
async function fetchNodeParameters(config, nodePubKey) {
    const nodeParams = await fetchDataFromNetwork(config, `/account/${nodePubKey}?type=9`, data => (data === null || data === void 0 ? void 0 : data.account) == null);
    if (nodeParams === null || nodeParams === void 0 ? void 0 : nodeParams.account) {
        if ('data' in nodeParams.account) {
            return nodeParams.account.data;
        }
        else {
            return nodeParams.account;
        }
    }
    else {
        return null;
    }
}
async function fetchNodeLoad(config) {
    const url = `http://localhost:${config.server.ip.externalPort}/load`;
    const nodeLoad = await axios_1.default.get(url);
    if ((nodeLoad === null || nodeLoad === void 0 ? void 0 : nodeLoad.data) == null) {
        throw new Error('Node not active in the network. Unable to fetch node load');
    }
    return nodeLoad.data;
}
async function fetchNodeTxStats(config) {
    const url = `http://localhost:${config.server.ip.externalPort}/tx-stats`;
    const txStats = await axios_1.default.get(url);
    if ((txStats === null || txStats === void 0 ? void 0 : txStats.data) == null) {
        throw new Error('Node not active in the network. Unable to fetch node tx-stats');
    }
    return txStats.data;
}
/**
 *
 * @param config node config to get url and port
 * @throws {AxiosError} Status 404. If unable to fetch node-info
 * @returns nodeInfo
 */
async function fetchNodeInfo(config) {
    const url = `http://localhost:${config.server.ip.externalPort}/nodeinfo?reportIntermediateStatus=true`;
    // const url = `http://localhost:${config.server.ip.externalPort}/nodeinfo`;
    const response = await axios_1.default.get(url, { timeout: 2000 });
    return response.data.nodeInfo;
}
exports.fetchNodeInfo = fetchNodeInfo;
async function fetchNetworkStats(config) {
    const networkStats = await fetchDataFromNetwork(config, '/network-stats', data => data == null);
    if (networkStats == null) {
        throw new Error('Unable to fetch network-stats');
    }
    return networkStats;
}
async function getNetworkParams(config, description) {
    try {
        const networkStats = await fetchNetworkStats(config);
        let result = { ...networkStats };
        // Check if description is undefined
        if (!description) {
            return result;
        }
        const status = (0, pm2_1.statusFromPM2)(description);
        if ((status === null || status === void 0 ? void 0 : status.status) && status.status !== 'stopped') {
            const nodeLoad = await fetchNodeLoad(config);
            const nodeTxStats = await fetchNodeTxStats(config);
            result = { ...result, ...nodeLoad, ...nodeTxStats };
        }
        return result;
    }
    catch (e) {
        console.error(e);
    }
    return null;
}
exports.getNetworkParams = getNetworkParams;
async function fetchEOADetails(config, eoaAddress) {
    const eoaParams = await fetchDataFromNetwork(config, `/account/${eoaAddress}`, data => (data === null || data === void 0 ? void 0 : data.account) == null);
    return eoaParams === null || eoaParams === void 0 ? void 0 : eoaParams.account;
}
exports.fetchEOADetails = fetchEOADetails;
async function fetchUnstakeableDetails(config, nominee, nominator) {
    const unstakable = await fetchDataFromNetwork(config, `/canUnstake/${nominee}/${nominator}`, data => (data === null || data === void 0 ? void 0 : data.stakeUnlocked) == null);
    return unstakable === null || unstakable === void 0 ? void 0 : unstakable.stakeUnlocked;
}
exports.fetchUnstakeableDetails = fetchUnstakeableDetails;
async function fetchValidatorVersions(config) {
    const validatorVersions = await fetchDataFromNetwork(config, '/nodeinfo', data => data == null);
    return validatorVersions === null || validatorVersions === void 0 ? void 0 : validatorVersions.nodeInfo.appData;
}
exports.fetchValidatorVersions = fetchValidatorVersions;
async function getAccountInfoParams(config, nodePubKey) {
    const params = {
        lockedStake: '',
        nominator: '',
        accumulatedRewards: new ethereumjs_util_1.BN(0),
        totalPenalty: '',
    };
    if (nodePubKey === '') {
        // Public key not found. This can happen in the primitive case when
        // the node has not been started for the first time.
        return params;
    }
    // prettier-ignore
    const { nodeRewardAmount, nodeRewardInterval } = await fetchInitialParameters(config);
    let nodeActiveDuration, previousRewards;
    try {
        const nodeData = await fetchNodeParameters(config, nodePubKey);
        if (!nodeData) {
            return params;
        }
        params.lockedStake = nodeData.stakeLock
            ? new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(nodeData.stakeLock), 16).toString()
            : '';
        previousRewards = new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(nodeData.reward), 16);
        const startTime = nodeData.rewardStartTime * 1000;
        const endTime = nodeData.rewardEndTime * 1000;
        if (startTime > 0 && endTime === 0) {
            // Node is earning rewards
            nodeActiveDuration = Date.now() - startTime;
            // Calculate rewards only if the node is active
            const totalReward = nodeRewardAmount.mul(new ethereumjs_util_1.BN(nodeActiveDuration, 16));
            params.accumulatedRewards = previousRewards.add(totalReward.div(nodeRewardInterval));
        }
        else {
            // Node is not earning rewards or has already earned rewards
            nodeActiveDuration = 0;
            params.accumulatedRewards = previousRewards;
        }
        params.nominator = nodeData.nominator;
        params.totalPenalty = new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(nodeData.nodeAccountStats.totalPenalty), 16).toString();
    }
    catch (err) {
        params.lockedStake = new ethereumjs_util_1.BN(0).toString();
        nodeActiveDuration = 0;
        params.nominator = '';
        previousRewards = new ethereumjs_util_1.BN(0);
        params.totalPenalty = new ethereumjs_util_1.BN(0).toString();
    }
    return params;
}
exports.getAccountInfoParams = getAccountInfoParams;
async function fetchStakeParameters(config) {
    const value = exports.cache.get('stakeParams');
    if (value) {
        return {
            stakeRequired: value,
        };
    }
    const stakeParams = await fetchDataFromNetwork(config, '/stake', data => !data);
    if (!stakeParams) {
        throw new Error("Couldn't fetch stake parameters");
    }
    const stakeRequired = new ethereumjs_util_1.BN((0, ethereumjs_util_1.stripHexPrefix)(stakeParams.stakeRequired.value), 16).toString();
    const cycleDuration = await fetchCycleDuration(config);
    exports.cache.set('stakeParams', stakeRequired, cycleDuration * 1000);
    return {
        stakeRequired,
    };
}
exports.fetchStakeParameters = fetchStakeParameters;
async function fetchCycleDuration(config) {
    const latestCycle = await fetchDataFromNetwork(config, '/sync-newest-cycle', data => !data);
    if (!latestCycle) {
        throw new Error("Couldn't fetch latest cycle");
    }
    return latestCycle.newestCycle.duration;
}
exports.fetchCycleDuration = fetchCycleDuration;
//# sourceMappingURL=fetch-network-data.js.map